@@ -2,10 +2,11 @@
 Traitlets based configuration for jupyter_server_proxy
 """
 from jupyter_server.utils import url_path_join as ujoin
-from traitlets import Dict, List, Union, default, observe
+from traitlets import Dict, List, Tuple, Union, default, observe
 from traitlets.config import Configurable
+from tornado import httpclient
 from warnings import warn
-from .handlers import SuperviseAndProxyHandler, AddSlashHandler
+from .handlers import SuperviseAndProxyHandler, AddSlashHandler, RewritableResponse
 import pkg_resources
 from collections import namedtuple
 from .utils import call_with_asked_args
@@ -16,7 +17,7 @@ try:
 except ImportError:
     from .utils import Callable
 
-def _make_serverproxy_handler(name, command, environment, timeout, absolute_url, port, mappath, request_headers_override):
+def _make_serverproxy_handler(name, command, environment, timeout, absolute_url, port, mappath, request_headers_override, rewrite_response):
     """
     Create a SuperviseAndProxyHandler subclass with given parameters
     """
@@ -29,6 +30,7 @@ def _make_serverproxy_handler(name, command, environment, timeout, absolute_url,
             self.absolute_url = absolute_url
             self.requested_port = port
             self.mappath = mappath
+            self.rewrite_response = rewrite_response
 
         @property
         def process_args(self):
@@ -74,11 +76,13 @@ def _make_serverproxy_handler(name, command, environment, timeout, absolute_url,
     return _Proxy
 
 
-def get_entrypoint_server_processes():
+def get_entrypoint_server_processes(serverproxy_config):
     sps = []
     for entry_point in pkg_resources.iter_entry_points('jupyter_serverproxy_servers'):
+        name = entry_point.name
+        server_process_config = entry_point.load()()
         sps.append(
-            make_server_process(entry_point.name, entry_point.load()())
+            make_server_process(name, server_process_config, serverproxy_config)
         )
     return sps
 
@@ -97,6 +101,7 @@ def make_handlers(base_url, server_processes):
             sp.port,
             sp.mappath,
             sp.request_headers_override,
+            sp.rewrite_response,
         )
         handlers.append((
             ujoin(base_url, sp.name, r'(.*)'), handler, dict(state={}),
@@ -108,9 +113,11 @@ def make_handlers(base_url, server_processes):
 
 LauncherEntry = namedtuple('LauncherEntry', ['enabled', 'icon_path', 'title', 'path_info'])
 ServerProcess = namedtuple('ServerProcess', [
-    'name', 'command', 'environment', 'timeout', 'absolute_url', 'port', 'mappath', 'launcher_entry', 'new_browser_tab', 'request_headers_override'])
+    'name', 'command', 'environment', 'timeout', 'absolute_url', 'port',
+    'mappath', 'launcher_entry', 'new_browser_tab', 'request_headers_override', 'rewrite_response',
+])
 
-def make_server_process(name, server_process_config):
+def make_server_process(name, server_process_config, serverproxy_config):
     le = server_process_config.get('launcher_entry', {})
     return ServerProcess(
         name=name,
@@ -127,7 +134,11 @@ def make_server_process(name, server_process_config):
             path_info=le.get('path_info', name + "/")
         ),
         new_browser_tab=server_process_config.get('new_browser_tab', True),
-        request_headers_override=server_process_config.get('request_headers_override', {})
+        request_headers_override=server_process_config.get('request_headers_override', {}),
+        rewrite_response=server_process_config.get(
+            'rewrite_response',
+            tuple(),
+        ),
     )
 
 class ServerProxy(Configurable):
@@ -173,16 +184,20 @@ class ServerProxy(Configurable):
 
             Keys recognized are:
 
-            enabled
-              Set to True (default) to make an entry in the launchers. Set to False to have no
-              explicit entry.
+              enabled
+                Set to True (default) to make an entry in the launchers. Set to False to have no
+                explicit entry.
 
-            icon_path
-              Full path to an svg icon that could be used with a launcher. Currently only used by the
-              JupyterLab launcher
+              icon_path
+                Full path to an svg icon that could be used with a launcher. Currently only used by the
+                JupyterLab launcher
 
-            title
-              Title to be used for the launcher entry. Defaults to the name of the server if missing.
+              title
+                Title to be used for the launcher entry. Defaults to the name of the server if missing.
+
+              path_info
+                The trailing path that is appended to the user's server URL to access the proxied server.
+                By default it is the name of the server followed by a trailing slash.
 
           new_browser_tab
             Set to True (default) to make the proxied server interface opened as a new browser tab. Set to False
@@ -192,9 +207,54 @@ class ServerProxy(Configurable):
             A dictionary of additional HTTP headers for the proxy request. As with
             the command traitlet, {{port}} and {{base_url}} will be substituted.
 
-          path_info
-            The trailing path that is appended to the user's server URL to access the proxied server.
-            By default it is the name of the server followed by a trailing slash.
+          rewrite_response
+            An optional function to rewrite the response for the given service.
+            Input is a RewritableResponse object which is an argument that MUST be named
+            ``response``. The function should modify one or more of the attributes
+            ``.body``, ``.headers``, ``.code``, or ``.reason`` of the ``response``
+            argument. For example:
+
+                def dog_to_cat(response):
+                    response.headers["I-Like"] = "tacos"
+                    response.body = response.body.replace(b'dog', b'cat')
+
+                c.ServerProxy.servers['my_server']['rewrite_response'] = dog_to_cat
+
+            The ``rewrite_response`` function can also accept several optional
+            positional arguments. Arguments named ``host``, ``port``, and ``path`` will
+            receive values corresponding to the URL ``/proxy/<host>:<port><path>``. In
+            addition, the original Tornado ``HTTPRequest`` and ``HTTPResponse`` objects
+            are available as arguments named ``request`` and ``orig_response``. (These
+            objects should not be modified.)
+
+            A list or tuple of functions can also be specified for chaining multiple
+            rewrites. For example:
+
+                def cats_only(response, path):
+                    if path.startswith("/cat-club"):
+                        response.code = 403
+                        response.body = b"dogs not allowed"
+
+                c.ServerProxy.servers['my_server']['rewrite_response'] = [dog_to_cat, cats_only]
+
+            Note that if the order is reversed to ``[cats_only, dog_to_cat]``, then accessing
+            ``/cat-club`` will produce a "403 Forbidden" response with body "cats not allowed"
+            instead of "dogs not allowed".
+
+            Defaults to the empty tuple ``tuple()``.
+        """,
+        config=True
+    )
+
+    non_service_rewrite_response = Union(
+        default_value=tuple(),
+        trait_types=[List(), Tuple(), Callable()],
+        help="""
+        A function (or list or tuple of functions) to rewrite the response for a
+        non-service request, for example a request to ``/proxy/<host>:<port><path>``.
+
+        See the description for ``rewrite_response`` for more information.
+        Defaults to the empty tuple ``tuple()``.
         """,
         config=True
     )
